/***********************************\
*                                   *
*  Dialog for editting canvas size  *
*                                   *
\***********************************/

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <math.h>

#include "canvassize.h"

#define RATIO_PIXELS 1
#define RATIO_CENTIMETERS (1/28.35)
#define RATIO_INCHES (1/72.0)

struct TCSState {
	GtkWidget *Window,*EWidth,*EHeight,*CBUnits,*CBConstrain;
	GtkWidget *MIPixels,*MICentimeters,*MIInches;
	void (*CallBack)(TCADSheetSize &,gpointer);
	gpointer UserData;

	TCADSheetSize size;
	double Ratio;

	gboolean ignoreChange;
};

static void
RefreshSizes(TCSState *s,gboolean refreshW,gboolean refreshH)
{
	double r;
	char buf[1024];

	DTP("RefreshSizes()\n");

	s->ignoreChange=true;

	switch(s->size.metric) {
	case GTK_PIXELS:
		r = RATIO_PIXELS;
		break;
	case GTK_CENTIMETERS:
		r = RATIO_CENTIMETERS;
		break;
	default:
		r = RATIO_INCHES;
	}

	if (refreshW) {
		if (r == 1) sprintf(buf,"%d",s->size.width);
		else sprintf(buf,"%.2f",s->size.width*r);
		gtk_entry_set_text(GTK_ENTRY(s->EWidth),buf);
	}

	if (refreshH) {
		if (r == 1) sprintf(buf,"%d",s->size.height);
		else sprintf(buf,"%.2f",s->size.height*r);
		gtk_entry_set_text(GTK_ENTRY(s->EHeight),buf);
	}
	
	s->ignoreChange=false;
}

static void
Close(TCSState *s)
{
	DTP("Close()\n");

  gtk_widget_hide(s->Window);
  gtk_grab_remove(s->Window);
	
	gtk_widget_destroy(s->Window);
	
	delete s;
}

/**********************************************************/

static void
on_EWidth_changed(GtkEditable *editable, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	double val,r;
	char *S;

	DTP("on_EWidth_changed()\n");

	if (s->ignoreChange) return;

	if (s->size.metric == GTK_PIXELS) r = RATIO_PIXELS;
	else if (s->size.metric == GTK_CENTIMETERS) r = RATIO_CENTIMETERS;
	else r = RATIO_INCHES;

	val = 0;
	if (sscanf(gtk_entry_get_text(GTK_ENTRY(editable)),"%lf",&val)) {
		val /= r;
		s->size.width = (int)rint(val);
		if (s->size.constrainRatio) {
			s->size.height = (int)rint(val/s->Ratio);
			RefreshSizes(s,false,true);
		}
	}
}

static void
on_EHeight_changed(GtkEditable *editable, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	double val,r;

	DTP("on_EHeight_changed()\n");

	if (s->ignoreChange) return;

	if (s->size.metric == GTK_PIXELS) r = RATIO_PIXELS;
	else if (s->size.metric == GTK_CENTIMETERS) r = RATIO_CENTIMETERS;
	else r = RATIO_INCHES;

	val = 0;
	if (sscanf(gtk_entry_get_text(GTK_ENTRY(editable)),"%lf",&val)) {
		val /= r;
		s->size.height = (int)rint(val);
		if (s->size.constrainRatio) {
			s->size.width = (int)rint(val*s->Ratio);
			RefreshSizes(s,true,false);
		}
	}
}

static void
on_CBConstrain_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	DTP("on_CBConstrain_toggled()\n");

	s->size.constrainRatio = gtk_toggle_button_get_active(togglebutton);
}

void
on_CBUnits_changed(GtkWidget *widget, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	GtkWidget *a = gtk_menu_get_active(GTK_MENU(s->CBUnits));

	DTP("on_CBUnits_changed()\n");

	if (a == s->MIPixels) s->size.metric = GTK_PIXELS;
	else if (a == s->MICentimeters) s->size.metric = GTK_CENTIMETERS;
	else s->size.metric = GTK_INCHES;

	RefreshSizes(s,true,true);
}

void
on_BOK_released(GtkButton *button, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	button = button;

	DTP("on_BOK_released()\n");

	if (s->CallBack) {
		(*(s->CallBack))(s->size,s->UserData);
	}

	Close(s);
}

void
on_BCancel_released(GtkButton *button, gpointer user_data)
{
	TCSState *s = (TCSState*)user_data;
	button = button;

	DTP("on_BCancel_released()\n");

	Close(s);
}


/**************************************************************/

void
CanvasSizeDialog(TCADSheetSize &size,void (*CallBack)(TCADSheetSize&,gpointer),gpointer UserData)
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *table1;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *label7;
  GtkWidget *EWidth;
  GtkWidget *EHeight;
  GtkWidget *CBConstrain;
  GtkWidget *CBUnits;
  GtkWidget *CBUnits_menu;
  GtkWidget *mi1,*mi2,*mi3;
  GtkWidget *hbox5;
  GtkWidget *BOK;
  GtkWidget *label8;
  GtkWidget *BCancel;
  GtkAccelGroup *accel_group;

	DTP("CanvasSizeDialog()\n");

	TCSState *State = new TCSState();

  window1 = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), "Sheet size");
  gtk_window_position (GTK_WINDOW (window1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (window1), TRUE, TRUE, FALSE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);
  gtk_container_border_width (GTK_CONTAINER (vbox1), 4);

  table1 = gtk_table_new (4, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (window1), "table1", table1);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

  label4 = gtk_label_new ("Canvas width: ");
  gtk_object_set_data (GTK_OBJECT (window1), "label4", label4);
  gtk_widget_show (label4);
  gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_RIGHT);

  label5 = gtk_label_new ("Canvas height: ");
  gtk_object_set_data (GTK_OBJECT (window1), "label5", label5);
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_RIGHT);

  label6 = gtk_label_new ("Units: ");
  gtk_object_set_data (GTK_OBJECT (window1), "label6", label6);
  gtk_widget_show (label6);
  gtk_table_attach (GTK_TABLE (table1), label6, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label6), GTK_JUSTIFY_RIGHT);

  label7 = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (window1), "label7", label7);
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table1), label7, 0, 1, 3, 4,
                    (GtkAttachOptions)( GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  EWidth = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "EWidth", EWidth);
  gtk_widget_show (EWidth);
  gtk_table_attach (GTK_TABLE (table1), EWidth, 1, 2, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_signal_connect_after (GTK_OBJECT (EWidth), "changed",
                            GTK_SIGNAL_FUNC (on_EWidth_changed),
                            State);

  EHeight = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "EHeight", EHeight);
  gtk_widget_show (EHeight);
  gtk_table_attach (GTK_TABLE (table1), EHeight, 1, 2, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_signal_connect_after (GTK_OBJECT (EHeight), "changed",
                            GTK_SIGNAL_FUNC (on_EHeight_changed),
                            State);

  CBConstrain = gtk_check_button_new_with_label ("Constrain ratio");
  gtk_object_set_data (GTK_OBJECT (window1), "CBConstrain", CBConstrain);
  gtk_widget_show (CBConstrain);
  gtk_table_attach (GTK_TABLE (table1), CBConstrain, 1, 2, 3, 4,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_signal_connect_after (GTK_OBJECT (CBConstrain), "toggled",
                            GTK_SIGNAL_FUNC (on_CBConstrain_toggled),
                            State);

  CBUnits = gtk_option_menu_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "CBUnits", CBUnits);
  gtk_widget_show (CBUnits);
  gtk_table_attach (GTK_TABLE (table1), CBUnits, 1, 2, 2, 3,
                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
  CBUnits_menu = gtk_menu_new ();
  mi1 = gtk_menu_item_new_with_label ("Pixels");
  gtk_signal_connect_after (GTK_OBJECT (mi1), "activate",
                            GTK_SIGNAL_FUNC (on_CBUnits_changed),
                            State);
  gtk_widget_show (mi1);
  gtk_menu_append (GTK_MENU (CBUnits_menu), mi1);
	
  mi2 = gtk_menu_item_new_with_label ("Centimeters");
  gtk_signal_connect_after (GTK_OBJECT (mi2), "activate",
                            GTK_SIGNAL_FUNC (on_CBUnits_changed),
                            State);
  gtk_widget_show (mi2);
  gtk_menu_append (GTK_MENU (CBUnits_menu), mi2);

  mi3 = gtk_menu_item_new_with_label ("Inches");
  gtk_signal_connect_after (GTK_OBJECT (mi3), "activate",
                            GTK_SIGNAL_FUNC (on_CBUnits_changed),
                            State);
  gtk_widget_show (mi3);
  gtk_menu_append (GTK_MENU (CBUnits_menu), mi3);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (CBUnits), CBUnits_menu);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "hbox5", hbox5);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox5, TRUE, TRUE, 0);

  BOK = gtk_button_new_with_label ("OK");
  gtk_object_set_data (GTK_OBJECT (window1), "BOK", BOK);
  gtk_widget_show (BOK);
  gtk_box_pack_start (GTK_BOX (hbox5), BOK, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (BOK), 4);
  gtk_signal_connect_after (GTK_OBJECT (BOK), "released",
                            GTK_SIGNAL_FUNC (on_BOK_released),
                            State);
  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW (window1), accel_group);
  gtk_widget_add_accelerator (BOK, "released", accel_group,
                              GDK_KP_Enter, 0, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator (BOK, "released", accel_group,
                              GDK_Return, 0, GTK_ACCEL_VISIBLE);

  label8 = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (window1), "label8", label8);
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox5), label8, TRUE, TRUE, 4);

  BCancel = gtk_button_new_with_label ("Cancel");
  gtk_object_set_data (GTK_OBJECT (window1), "BCancel", BCancel);
  gtk_widget_show (BCancel);
  gtk_box_pack_start (GTK_BOX (hbox5), BCancel, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (BCancel), 4);
  gtk_signal_connect_after (GTK_OBJECT (BCancel), "released",
                            GTK_SIGNAL_FUNC (on_BCancel_released),
                            State);
  gtk_widget_add_accelerator (BCancel, "released", accel_group,
                              GDK_Escape, 0, GTK_ACCEL_VISIBLE);

	State->Window = window1;
	State->EWidth = EWidth;
	State->EHeight = EHeight;
	State->CBUnits = CBUnits_menu;
	State->CBConstrain = CBConstrain;
	State->UserData = UserData;
	State->MIPixels = mi1;
	State->MICentimeters = mi2;
	State->MIInches = mi3;
	State->size = size;
	State->ignoreChange=false;
	State->CallBack = CallBack;

	if ((size.width <= 0) || (size.height <= 0)) State->Ratio = 0;
	else State->Ratio = size.width/size.height;

	// Constrain ratio
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CBConstrain),(size.constrainRatio));

	// Unit combo
	switch(size.metric) {
	case GTK_PIXELS:
		gtk_option_menu_set_history(GTK_OPTION_MENU(CBUnits),0);
		break;
	case GTK_CENTIMETERS:
		gtk_option_menu_set_history(GTK_OPTION_MENU(CBUnits),1);
		break;
	case GTK_INCHES:
		gtk_option_menu_set_history(GTK_OPTION_MENU(CBUnits),2);
		break;
	}

	// Refresh sizes
	RefreshSizes(State,true,true);

  gtk_widget_show(window1);
  gtk_grab_add(window1);
}


