/*********\
*         *
*  Sheet  *
*         *
\*********/

#ifndef GTKFIG_SHEET_H
#define GTKFIG_SHEET_H

#include <gtk/gtkvruler.h>
#include <gtk/gtkhruler.h>
#include <gtk/gtkframe.h>
#include "group.h"

class TCADView;

/*
 * Alignments of ends of lines 
 */
typedef enum { LAL_NONE, LAL_GRID, LAL_HALFGRID } TLineAlignment;

/*
 * For alignment of objects
 */
typedef enum { VAL_NONE, VAL_GRID, VAL_TOPS, VAL_BOTTOMS, VAL_SAMEHEIGHT } TVertAlign;
typedef enum { HAL_NONE, HAL_GRID, HAL_LEFTS, HAL_RIGHTS, HAL_SAMEWIDTH } THorizAlign;

/*
 * Result from ButtonPress
 */
typedef enum { BPR_NOTHING, BPR_OBJECT, BPR_HANDLE } TBPResult;

/*
 * Initial size of canvas
 */
#define INITIAL_WIDTH 1000
#define INITIAL_HEIGHT 1000

/*
 * Sheet size info
 */
class TCADSheetSize {
public:
	int width,height;
	gboolean constrainRatio;
	GtkMetricType metric;

	TCADSheetSize(int w=INITIAL_WIDTH,int h=INITIAL_HEIGHT,gboolean c=true,GtkMetricType m=GTK_PIXELS):width(w),height(h),constrainRatio(c),metric(m) {}
  virtual void Save(TCADSaveStream *Fl);
  virtual void SaveProperties(TCADSaveStream *Fl);
  virtual char LoadProperty(TCADLoadStream *Fl,char *ID);
  virtual void Load(TCADLoadStream *Fl);
};

/*
 * Represents one sheet
 */
class TCADSheet : public TCADGroup {
protected:
  unsigned long LastUniq;

  TCADObject *FirstSelected;

  // Updates FocusRect to contain rectangle (x,y,DownX,DownY) and draws it.
  void UpdateFocusRect(int x,int y);

  // Grid
  gboolean UseGrid,ShowGrid,GX_eq_GY;
  int GridX,GridY;
  gboolean Toggling;

  GdkRectangle WS;  // Contains window position and size for loading

	// Canvas size
	TCADSheetSize size;

  // Do not change EncapRect of sheet
  virtual void ComputeEncapRect() { }
  virtual void AffectEncapRect(TCADObject *Child) { Child=Child; }

	// To be called from PostLoad() and SetSheetSize()
	void ApplySheetSize();

	// Undo & changes
	unsigned int SavedCount;  // Change count when we last saved
	unsigned int ChangeCount; // Increments with every change
	enum { MAX_UNDO_LEVELS=21 }; 	// It is actually one higher than "real" value, because we save current state
	TCADMemory *UndoMem[MAX_UNDO_LEVELS];
	int UMFirst,UMLast,UMAct;
	void SaveState();         // Saves state for undo/redo
	
	// Gets base title
	char *GetTitle();
public:
  TCADSheet(char Visible);
  ~TCADSheet();
  virtual void Save(TCADSaveStream *Fl);
  virtual void SaveProperties(TCADSaveStream *Fl);
  virtual void Load(TCADLoadStream *Fl);
  virtual char LoadProperty(TCADLoadStream *Fl,char *ID);
  virtual void PostLoad(TCADLoadStream *Fl);
  virtual void Export(char *FileName,TExportFormat Format);

  // Views
  TCADView *FirstView;

  // Filename or NULL for new sheets
  char *FileName;

  gboolean HasFocus();  // Returns if any view has focus

  // Shows all views
  void Show();

  // Hides all views
  void Hide();

  // Redraws sheet in all views
  void DrawRect(GdkRectangle *Rect);

  // Redraws sheet in one view
  void DrawRect(TCADView *View,GdkRectangle *ORect,int MagA,int MagB);

  // Retrieves next uniq number
  unsigned long NextUniq();

  // Returns TRUE when object/handle was clicked, otherwise FALSE.
  virtual TBPResult ButtonPressed(int x,int y,gboolean Shift,TCADView *View);
//  virtual void ButtonReleased();
//  virtual void MouseMoved(int x,int y);

  // Creating of objects
  TCADLine *CreateLine(int Type,int x,int y);
  void NewRectObject(int Kind,GdkRectangle *Bounds);

  // Selects objects in the rectangle
  void SelectRectangle(int x1,int y1,int x2,int y2);

  // Moves selected objects
  void MoveSelected(int dx,int dy);

  void UnselectAll();
  void SelectAll();
  void SelectObject(TCADObject *Object,GdkRectangle *RefreshRect,GdkRectangle *RefreshRect2);

  void AlignObjects(THorizAlign HA, TVertAlign VA);

  void DeleteSelected();
  void EditText(gboolean ModifyOnCancel);

	void Undo();
	void Redo();
	void EditCut();
	void EditCopy();
	void EditPaste();
	
  TCADObject *GetFirstSelected();

  void ShowArrowProperties();
  void ShowTextProperties();
  void ShowLineProperties();
  void ShowGridProperties();

  // Sets title according to filename
  void SetTitle();

  gboolean GetShowGrid() { return ShowGrid; }
  gboolean GetUseGrid() { return UseGrid; }
  void GetGridOptions(gboolean& X_eq_Y,int& GX,int& GY);
  void SetShowGrid(gboolean Value);
  void SetUseGrid(gboolean Value);
  void SetGridOptions(gboolean X_eq_Y,int GX,int GY);

  // View routines
  void AddView(TCADView *);	// Informs sheet that view has been added

	// Returns TRUE when we should destroy the sheet, FALSE when we should not.
  gboolean RemoveView(TCADView *);  

  // Returns true if we can close, false when dialog was shown
  gboolean Close();

	// Boolean saying we are closing
	gboolean Closing;

	// Gets/sets width & height of the sheet
	void GetSheetSize(int &Width,int &Height);
	void SetSheetSize(TCADSheetSize &size);

	// Show dialog to set canvas width/height
	void SheetSizeDialog();

	void BringToFront();
	void SendToBack();

	// Changes
	void Changed();
	void Saved();
	gboolean IsModified() { return(SavedCount!=ChangeCount); }
	int BlockChange;  // Set to > 0 to ignore changes. Used in block operations
};


#endif

